/*
 * File:   clog.hh
 * Author: Balint Morvai
 *
 * Modified on Jan 2013
 *
 * Usage: 	CLogfile and CLogstream are intended to be used as slots for
 * 			the "boost signals2" library, via the () operator functions.
 * 			Therefore they are connected to signals2 slots in "log.hh".
 * 
 * 			CLogfile and CLogstream are NOT intended for external usage.
 *
 * MODIFIED BSD LICENSE:
 * 
 * Copyright (c) 2013, Balint Morvai
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the "Logger" nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef CLOG_HH
#define	CLOG_HH

#include "log_macros.hh"

#ifndef MPIHELPER
#define MPIHELPER
// DUMMY class used below to get processor id!
// REPLACE THIS CLASS WITH YOUR OWN MPI HELPER CLASS THAT
// WILL RETURN CORRECT RANK.
// BE SURE TO DO IT ALSO IN FILE "log.hh"!
class MPIhelper
{
	MPIhelper(){};
	~MPIhelper(){};
	
	int rank()
	{
		return 1;
	};
};
#endif

/** 
 * Offers a file stream, "init" has to be called before the stream can be used. 
 */
class CLogfile
{
	private:
	std::ofstream *logout_f;
	
	public:
	CLogfile(std::ofstream *logout_f_)
		:logout_f(logout_f_)
	{};
	
	CLogfile& operator<<(std::ostream& (*pf)(std::ostream&))
	{
		*logout_f << pf;
		return *this;
	};

	template < typename T >
	CLogfile& operator<<( T const & t )
	{
		*logout_f << t;
		return *this;
	};
	
	// has to be called before stream can be used
	void init( MPIhelper *helper, const std::string filename )
	{
		logout_f->close();
		logout_f->open( filename.c_str() );

		*logout_f << "Logfile for process " << helper->rank() << std::endl;
	};
	
	// DEPRECATED version for backward compatibility
	void init( const std::string filename )
	{
		logout_f->close();
		logout_f->open( filename.c_str() );
	};

	// write a string message to the stream
	void operator() ( const std::string msg )
	{
		*logout_f << msg << std::endl;
	};
	
	// write a string message to the stream
	void slotf( const std::string msg )
	{
		*logout_f << msg;
	};
	
	~CLogfile()
	{
		logout_f->close();
	};
};

/** 
 * Offers a string stream, "init" has to be called before the stream is used. 
 */
class CLogstream
{
	private:
	std::ostream *logout_s;
	
	public:
	CLogstream(std::ostream *logout_s_)
		:logout_s(logout_s_)
	{};

	CLogstream& operator<<(std::ostream& (*pf)(std::ostream&))
	{
		*logout_s << pf;
		return *this;
	};

	template < typename T >
	CLogstream& operator<<( T const & t )
	{
		*logout_s << t;
		return *this;
	};

	void init( MPIhelper *helper )
	{
		*logout_s << "Logstream for process " << helper->rank() << std::endl;
	};	

	// write a string message to the stream
	void operator() ( const std::string msg )
	{
		*logout_s << msg << std::endl;
	};
	
	// write a string message to the stream
	void slots( const std::string msg )
	{
		*logout_s << msg;
	};

	~CLogstream() 
	{};
};

#endif // CLOG_HH
