#ifndef LOG_MACROS_HH
#define LOG_MACROS_HH

// define logfile trunk name
#define LOG_FILE_NAME "Logfile_P"
// define logfile extension
#define LOG_FILE_EXT ".log"
// define the (boost chrono) time format for the log.
// "%H:%M%:%S" would produce "22:36:59"
#define LOG_TIME_FORMAT "%Y-%m-%d  %H:%M:%S"
// define the string preceding the processor rank
#define LOG_RANK_STRING "pid "
// define the log separator string used between time, pid and msg
#define LOG_SEPARATOR " - "

// define log output type macros - used to set output for Log
// that can go to Log or CLogfile or CLogstream
// Note: "i" is the "show info" modifier, while "p" is the "ignore PID"
//		 modifier. (PID is ignored on the out NOT followed by "p")
//		 the "double p" cases "OFpSpi" & "OFpSp" are equivalent to not 
//		 setting "p" at all, thus to "OFSi" & "OFS"
enum logout
{
	OFSi,	// log output to Log / to file and stream 	+ info
	OFS,	// log output to Log / to file and stream
    OFpSi,  // log output to Log / to file and stream	+ info	(writing on special pid on the stream will be ignored)
    OFSpi,  // log output to Log / to file and stream	+ info	(writing on special pid on the file will be ignored)
    OFpSpi, // log output to Log / to file and stream	+ info	(the same as OFSi (default))
    OFpS,   // log output to Log / to file and stream			(writing on special pid on the stream will be ignored)
    OFSp,   // log output to Log / to file and stream			(writing on special pid on the file will be ignored)
    OFpSp,  // log output to Log / to file and stream			(the same as OFS (default))
	OFi,	// log output to CLogfile 	/ to file		+ info
	OF,		// log output to CLogfile 	/ to file		
	OSi,	// log output to CLogstream / to stream		+ info
	OS		// log output to CLogstream / to stream		
};

// operator to deactivate info <=> fall back to case without "i"
inline logout& operator-- (logout &out, int)
{
	switch(out)
	{
		case OFSi: 		out = OFS; 		break;
		case OFpSi: 	out = OFpS; 	break;
		case OFSpi: 	out = OFSp; 	break;
		case OFpSpi: 	out = OFpSp; 	break;
		case OFi: 		out = OF; 		break;
		case OSi: 		out = OS; 		break;
		default: break; // just to satisfy compiler (no warning)
	}
		
	return out;
};

// checks whether out1 and out2 are equal EXCEPT for small letters
inline bool is_equal (logout out1, logout out2)
{
	int case1;
	int case2;
		
	switch(out1)
	{
		case OFS: case OFSi: case OFpSi: case OFSpi: case OFpSpi: case OFpS: case OFSp:
			case1 = 1;
			break;
		case OF: case OFi:
			case1 = 2;
			break;
		case OS: case OSi:
			case1 = 3;
			break;
	}
	switch(out2)
	{
		case OFS: case OFSi: case OFpSi: case OFSpi: case OFpSpi: case OFpS: case OFSp:
			case2 = 1;
			break;
		case OF: case OFi:
			case2 = 2;
			break;
		case OS: case OSi:
			case2 = 3;
			break;
	}
	
	return (case1 == case2);
};

// checks whether out contains an "i".
inline bool is_i (logout out)
{
	bool retval;
	
	switch(out)
	{
		case OFSi: case OFpSi: case OFSpi: case OFpSpi: case OFi: case OSi:
			retval = true;
			break;
		case OF: case OS: case OFS: case OFpS: case OFSp:
			retval = false;
			break;
	}
	
	return retval;
};

#endif
